#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['datasurveyor']

package_data = \
{'': ['*']}

install_requires = \
['pandas >=1.0.4']

extras_require = \
{'test': ['pytest >=5.4.3', 'pytest-cov >=2.10.0']}

setup(name='datasurveyor',
      version='0.0.1',
      description='Data exploration tools.',
      author='Nick Buker',
      author_email='nickbuker@gmail.com',
      url='https://github.com/nickbuker/datasurveyor',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='~=3.6,<4',
     )
